%% Code for Day 4

% Add path to EEGLab and launch EEGLab before we begin (or use your
% shortcut on the shortcuts bar)

% Make sure you can access your sample data (sampleEEGdata.mat)


%% --------------------------------------
% Making Sine Waves
% ---------------------------------------

% define some variables
freq = 2;         % frequency in Hz
time = -1:.001:1; % -1 to 1 seconds in millisecond steps

% now create the sinewave
sinewave = 1*sin(2*pi*freq*time + 0);

% now plot it!
figure;plot(time,sinewave);

%% Summing sine waves to make a complicated time series

% create a few sine waves and sum

% define a sampling rate
srate = 1000;

% list some frequencies (I used linspace from about 1 Hz to 50 Hz, 20 sinewaves)
frex = [linspace(1,50,20)];  

% list some amplitudes... make sure there are the same number of
% amplitudes as there are frequencies!  (I used 1/f which mimics many natural processes
amplit = 1./frex;

%% ASSIGNMENT -----------------------------------
% Plot amplitude (y axis) by frequency (x axis)


% phases... list some random numbers between -pi and pi
rand_denominator = rand(1,length(frex)); % make the right number for your frequencies
scale_factor = 2*pi; % range of phases  
rand_denominator = scale_factor*(rand_denominator -mean(rand_denominator)); % zero-mean 
phases = pi./rand_denominator;

% define time... (from -1 to 1 second at sample rate)
time = -1:1/srate:1;


% now we loop through frequencies and create sine waves
sine_waves = zeros(length(frex),length(time));
for fi=1:length(frex)
    sine_waves(fi,:) = amplit(fi) * sin(2*pi*time*frex(fi) + phases(fi));
end

%% ASSIGNMENT -----------------------------------
% Create figure with two subplots
% Plot all sine waves super-imposed in subplot 1
% Sum these waveforms to plot the resultant signal in subplot 2
% Make sure time is on the x axis

% OPTIONAL ASSIGNMENT IF DONE-------------------
% Change the number of sine waves, the frequencies and amplitudes and 
% phases of those sine waves(make your own) and re-run to 
% create a complex waveform


%% --------------------------------------
% Dot Product
% ---------------------------------------

% This cell from chapter 10 of the book code

% two vectors of random numbers
a = randn(10,1);
b = randn(10,1);

% initialize temporary matrix.
pointwise_result=zeros(size(a));

for i=1:length(a)
    pointwise_result(i) = a(i) * b(i);
end
dotproduct = sum(pointwise_result);


% The above code is useful if you are unfamiliar with 
% how a dot product works. Following is a bit more elegant: 
dotproduct = sum(a.*b);

% The most elegant way to compute the dot product, 
% however, is to use Matlab's short-cut: 
dotproduct = a'*b;

% This requires the first vector to be a row vector and the 
% second vector to be a column vector. Otherwise, it will 
% either crash (a*b) or give you the outer product (a*b'). 
% When in doubt, use sum(a.*b). 


%% --------------------------------------
% Convolution - the moving dot product!
% ---------------------------------------

% Make sine waves from 1 to 10 Hz, all starting at phase 0
srate = 1000; % define a sampling rate
frex = [linspace(1,10,10)];  % 10 frequencies
amplit = ones(size(frex)); % no scaling
phases = rand(size(frex)).*pi; % a bit of randomness
time = -1:1/srate:1; % define time... (from -1 to 1 second at sample rate)

% now we loop through frequencies and create sine waves
sine_waves2 = zeros(length(frex),length(time));
for fi=1:length(frex)
    sine_waves2(fi,:) = amplit(fi) * sin(2*pi*time*frex(fi) + phases(fi));
end

% make signal the sum of these
signal = sum(sine_waves2);

% devine kernel to be sinewave of freq f
f = 1;  % change as you wish!
kernel = sin(2*pi*time*f);

% use function conv that does dot product shifting the kernel across the signal
conv_result = conv(signal,kernel,'same');

%% ASSIGNMENT -----------------------------------
% Create figure with three subplots
% Plot signal subplot 1
% Plot kernel in subplot 2
% Plot conv_result in subplot 3

% Then change frequency of kernel to see result
% Then replot the figure to show the original waveform that is the same
% frequency as the kernel.  What do you observe?

%% Now explore FilterDemo 
edit FilterDemo.m


%% --------------------------------------
% DFT and FFT
% ---------------------------------------

% This code from Mike Cohen's Lecturelet

% Compute discrete-time Fourier transform

signal  = sum(sine_waves,1); % sum to simplify
N       = length(signal);    % length of sequence
fourierTime = ((1:N)-1)/N;   % "time" used for sine waves
nyquist = srate/2;           % Nyquist frequency -- the highest frequency you can measure in the data

% initialize Fourier output matrix
fourierCoefs = zeros(size(signal)); 

% These are the actual frequencies in Hz that will be returned by the
% Fourier transform. The number of unique frequencies we can measure is
% exactly 1/2 of the number of data points in the time series (plus DC). 
frequencies = linspace(0,nyquist,floor(N/2)+1);


% loop over frequencies
for fi=1:N
    
    % create sine wave for this frequency
    fourierSine = exp( -1i*2*pi*(fi-1).*fourierTime );
    
    % compute dot product as sum of point-wise elements
    fourierCoefs(fi) = sum(fourierSine.*signal);
    
    % note: this can also be expressed as a vector-vector product
    % fourier(fi) = fourierSine*signal';
end

% scale Fourier coefficients to original scale
fourierCoefs = fourierCoefs / N;


figure(1), clf
subplot(221)
plot(real(exp( -2*pi*1i*(10).*fourierTime )))
xlabel('time (a.u.)'), ylabel('Amplitude')
title('Sine wave')

subplot(222)
plot(signal)
title('Data')

subplot(212)
plot(frequencies,abs(fourierCoefs(1:length(frequencies)))*2,'*-')
xlabel('Frequency (Hz)')
ylabel('Power (\muV)')
title('Power spectrum derived from discrete Fourier transform')

%% Compare manual Fourier transform with fft

% Compute fourier transform and scale
fourierCoefsF = fft(signal) / N;

subplot(212),hold on
plot(frequencies,abs(fourierCoefsF(1:length(frequencies)))*2,'ro')
set(gca,'xlim',[0 40])

%% Compare computation times

% initialize variable for computation times
computationTimes = zeros(1,2);

% start Matlab's timer
tic;

% manual DTFT
for fi=1:N
    fourierSine = exp( -1i*2*pi*(fi-1).*fourierTime );
    fourierCoefs(fi) = sum( fourierSine .* signal );
end
fourierCoefs = fourierCoefs / N;

% end Matlab's timer
computationTimes(1) = toc;


% now FFT
tic;
fourierCoefsF = fft(signal) / N;
computationTimes(2) = toc;


figure(2), clf

bar(computationTimes)
set(gca,'xtick',1:2,'xticklabel',{'Manual';'fft'},'xlim',[0 3])
ylabel('Computation time in seconds')


%% Compute inverse discrete-time Fourier transform

reconSignal = zeros(size(signal));

% loop over frequencies
for fi=1:N
    
    % create sine wave for this frequency
    fourierSine = fourierCoefs(fi) * exp( 1i*2*pi*(fi-1).*fourierTime );
    
    % compute dot product as sum of point-wise elements
    reconSignal = reconSignal + fourierSine;
end

% note: in practice, the inverse Fourier transform should be done using:
reconSignal = ifft(fourierCoefs) * N;

figure(3), clf
plot(real(reconSignal),'.-')
hold on
plot(signal,'ro')

legend({'reconstructed';'original'})




%% --------------------------------------
% Obtaining the right frequencies from the FFT, windowing, stationarity
% ---------------------------------------
% Define FFT parameters
if length(signal) > 20*srate
    fft_points = length(signal);
else
    fft_points = srate*20; % 20-second fft length (better spectral resolution)
end

nSeries = 1;
nPoints = fft_points;

%% Windowing
hamming_window = hamming(length(signal));
figure; plot(hamming_window);

% Apply window
time_series = signal'.*hamming_window;
figure; plot(signal); 
hold on
plot(time_series,'-r');

%% Compute FFT, power
% Take fft
FFTX = fft(time_series,fft_points); 
% Calculate the number of unique points 
NumUniquePts = ceil((nPoints+1)/2); 
% FFT is symmetric, throw away second half 
FFTX = FFTX(1:NumUniquePts,:); 
% Take the magnitude of fft of x 
MX = abs(FFTX); 
% Scale the fft so that it is not a function of the length of x 
MX = MX./nPoints; 
% Take the square of the magnitude of fft of x. 
pwr = MX.^2; 
% Multiply by 2 because you threw out second half of FFTX above 
pwr = pwr.*2; 
% DC Component should be unique, i.e. undo multiply by 2. 
pwr(1,:) = pwr(1,:)./2; 
% Nyquist component should also be unique.
if ~rem(nPoints,2) 
   % Here NFFT is even; therefore, Nyquist point is included. 
   pwr(end,:) = pwr(end,:)./2; 
end 

%% Compute frequency vector
% This is an evenly spaced frequency vector with NumUniquePts points. 
fs = (0:NumUniquePts-1)* srate / nPoints;
fs=fs';

%% Plot the power spectrum
figure; plot(fs,pwr);


%% ASSIGNMENT -----------------------------------
% use the function FFT_Power to examine impact of nonstationarity and plot
% the results 

% use windowing for this first set of exercises

% first use the function to take the FFT of your signal

% now do this for a spike
spike = zeros(size(signal));
spike(round(length(signal)/2)) = 1;
%now plot the result

% now for a spike superimposed on the signal
spikey_signal = signal;
spikey_signal(round(length(signal)/2)) = 100;
%now plot the result

% Now superimpose the original power spectrum of the signal over the spikey
% signal power spectrum


% Now explore the impact of using the window versus not using the window
% Superimpose the power spectrum with and without windowing
% Note that they are not on the same scale.  Normalize each by max power
% and plot again.  What is the impact of windowing?


%% --------------------------------------
% Morlet wavelets and convolution
% ---------------------------------------
% get sample data
load SampleEEGdata.mat

srate = EEG.srate; % in hz
time  = -2:1/srate:2; % best practice is to have time=0 at the center of the wavelet
frex  = 6.5; % frequency of wavelet, in Hz

% create complex sine wave
sine_wave = exp( 1i*2*pi*frex.*time );

% create Gaussian window
s = 7 / (2*pi*frex); % this is the standard deviation of the gaussian
gaus_win  = exp( (-time.^2) ./ (2*s^2) );


% now create Morlet wavelet
cmw = sine_wave.*gaus_win;


figure(1), clf

subplot(211)
plot(time,real(cmw))
xlabel('Time (s)'), ylabel('Amplitude')
title([ 'Real part of wavelet at ' num2str(frex) ' Hz' ])

subplot(212)
plot(time,imag(cmw))
xlabel('Time (s)'), ylabel('Amplitude')
title([ 'Imaginary part of wavelet at ' num2str(frex) ' Hz' ])


figure(2), clf
plot3(time,real(cmw),imag(cmw),'linew',3)
axis image
xlabel('Time (s)'), ylabel('Real part'), zlabel('Imaginary part')
rotate3d

%% Morlet wavelet in the frequency domain

[fs pwr] = FFT_Power(real(cmw),srate,false);
figure; plot(fs,pwr);
xlabel('Frequency (Hz)'), ylabel('Amplitude')
set(gca,'xlim',[0 frex*4])

%% convolve wavelet on sample data first epoch, first channel
signal = EEG.data(1,:,1);
figure; subplot 211; plot(EEG.times,signal); 
waveletized_signal = conv(signal,cmw,'same');
subplot 212; plot(EEG.times,real(waveletized_signal));


%% ASSIGNMENT -----------------------------------
% Wavelet convolution and then Time Domain Averaging and plotting

% Create a family of Morlet wavelets ranging in frequency from 2 Hz to 30 Hz in five steps. 
% Select one electrode from the scalp EEG dataset and convolve each wavelet with EEG data from all trials from that electrode. 
% Apply the Matlab function real to the convolution result, as in convol_result=real(convol_result). This will return the EEG data bandpass filtered at the peak frequency of the wavelet. 
% Average the result of convolution over all trials and plot an ERP corresponding to each wavelet frequency. 
% Each frequency should be in its own subplot. 
% Plot the broadband ERP (without any convolution). 
% Thus, you will have six subplots in one figure. 
% How do the wavelet-convolved ERPs compare with the broadband ERP? Are there dynamics revealed by the wavelet-convolved ERPs that are not apparent in the broadband ERP, and are there dynamics in the broadband ERP that are not apparent in the waveletconvolved ERPs? Base your answer on qualitative visual inspection of the results; statistics or other quantitative comparisons are not necessary.
% Select five time points at which to show topographical plots of raw data (e.g., 0 to 400 ms in 100-ms steps). 
% Plot the same five points for wavelet-convolved data
% ADVANCED: Make a spiffy plot with rows corresponding to the subplots
% above, and columns to the timepoints, with a topoplot in each

